#include <msp/builder/builder.h>
#include <msp/builder/sourcepackage.h>
#include <msp/builder/tool.h>
#include <msp/fs/utils.h>
#include "datapackcomponent.h"
#include "datasourcefile.h"

using namespace std;
using namespace Msp;

DataPackComponent::DataPackComponent(SourcePackage &p, const string &n):
	Component(p, n)
{ }

void DataPackComponent::create_targets() const
{
	Builder &builder = package.get_builder();
	Tool &dcomp = builder.get_toolchain().get_tool("DATA");

	vector<Target *> files;
	for(const FS::Path &s: collect_source_files())
	{
		string ext = FS::extpart(FS::basename(s));
		if(ext==".mdt")
		{
			Target *src = dcomp.create_source(*this, s);
			files.push_back(dcomp.create_target(*src, "collection"));
		}
		else if(Target *tgt = builder.get_vfs().get_target(s))
			files.push_back(tgt);
		else
			files.push_back(new DataSourceFile(builder, *this, s));
	}

	Target *result = dcomp.create_target(files, "pack");

	BuildGraph &build_graph = builder.get_build_graph();
	build_graph.add_primary_target(*result);
	if(install)
		build_graph.add_installed_target(*result);
}
