#include <msp/builder/builder.h>
#include <msp/builder/cache.h>
#include <msp/builder/component.h>
#include <msp/builder/file.h>
#include <msp/builder/sourcepackage.h>
#include <msp/fs/dir.h>
#include <msp/fs/stat.h>
#include <msp/fs/utils.h>
#include <msp/strings/regex.h>
#include "datatransform.h"

using namespace std;
using namespace Msp;

DataTransform::DataTransform(Builder &b, const Component &c, const FS::Path &p):
	FileTarget(b, c.get_package(), p)
{
	component = &c;

	if(FS::Stat st = FS::lstat(FS::dirname(path)))
		dir_mtime = st.get_modify_time();
}

void DataTransform::find_dependencies()
{
	vector<string> files;
	Cache &cache = component->get_package().get_cache();
	const Time::TimeStamp &cache_mtime = cache.get_mtime();
	if(mtime<cache_mtime && dir_mtime<cache_mtime && cache.has_key(this, "files"))
		files = cache.get_values(this, "files");
	else
	{
		builder.get_logger().log("files", "Reading imports from %s", path.str());
		IO::File in(path.str());
		DataFile::Parser parser(in, path.str());

		vector<string> dir_files;
		while(!in.eof())
		{
			DataFile::Statement st = parser.parse();
			if(st.keyword=="for_each")
			{
				// There's bound to be at least one file: the transform itself
				if(dir_files.empty())
				{
					FS::Path dir = FS::dirname(path);
					builder.get_logger().log("files", "Traversing %s", dir.str());
					dir_files = list_files(dir);
				}

				for(const DataFile::Value &a: st.args)
				{
					Regex re(a.get<string>());
					for(const string &f: dir_files)
						if(re.match(f))
							files.push_back(f);
				}
			}
			else if(st.keyword=="file" && st.args.size()==1)
				files.push_back(st.args.front().get<string>());
		}

		cache.set_values(this, "files", files);
	}

	for(const string &f: files)
	{
		FS::Path file_path = FS::dirname(path)/f;
		if(Target *tgt = builder.get_vfs().get_target(file_path))
			add_transitive_dependency(*tgt);
		else
			add_transitive_dependency(*new File(builder, *package, file_path));
	}
}
