#include <stdexcept>
#include <msp/builder/builder.h>
#include <msp/builder/component.h>
#include <msp/builder/objectfile.h>
#include <msp/builder/sourcepackage.h>
#include <msp/builder/staticlibrary.h>
#include <msp/fs/path.h>
#include <msp/fs/utils.h>
#include "gnuarchiver.h"

using namespace std;
using namespace Msp;

GnuArchiver::GnuArchiver(Builder &b, const Architecture &a):
	Tool(b, &a, "AR")
{
	set_command("ar", true);
	input_suffixes.push_back(".o");
	processing_unit = COMPONENT;
	set_run_external(_run);
}

Target *GnuArchiver::create_target(const vector<Target *> &sources, const string &)
{
	if(sources.empty())
		throw invalid_argument("GnuArchiver::create_target");

	vector<ObjectFile *> objs;
	objs.reserve(sources.size());
	for(Target *s: sources)
		objs.push_back(&dynamic_cast<ObjectFile &>(*s));

	const Component &comp = *objs.front()->get_component();
	StaticLibrary *lib = new StaticLibrary(builder, comp, objs);
	lib->set_tool(*this);
	return lib;
}

ExternalTask::Arguments GnuArchiver::_run(const StaticLibrary &lib, FS::Path &work_dir)
{
	const Tool &tool = *lib.get_tool();

	vector<string> argv;
	argv.push_back(tool.get_executable()->get_path().str());
	argv.push_back("rc");

	argv.push_back(relative(lib.get_path(), work_dir).str());

	for(Target *d: lib.get_dependencies())
		if(ObjectFile *obj = dynamic_cast<ObjectFile *>(d))
			argv.push_back(relative(obj->get_path(), work_dir).str());

	return argv;
}
