#include <cstdlib>
#include <msp/builder/builder.h>
#include <msp/builder/component.h>
#include <msp/builder/exportdefinitions.h>
#include <msp/builder/externaltask.h>
#include <msp/builder/importlibrary.h>
#include <msp/builder/installedfile.h>
#include <msp/builder/objectfile.h>
#include <msp/builder/sharedlibrary.h>
#include <msp/builder/sourcepackage.h>
#include <msp/fs/utils.h>
#include <msp/strings/format.h>
#include "mingwdlltool.h"

using namespace std;
using namespace Msp;

MingwDllTool::MingwDllTool(Builder &b, const Architecture &a):
	Tool(b, &a, "DLL")
{
	set_command("dlltool", true);
	set_run(_run);
}

Target *MingwDllTool::create_target(const vector<Target *> &sources, const string &)
{
	if(sources.size()!=1)
		throw invalid_argument("MingwDllTool::create_target");
	SharedLibrary &shlib = dynamic_cast<SharedLibrary &>(*sources.front());

	vector<ObjectFile *> objs;
	objs.reserve(shlib.get_dependencies().size());
	for(Target *d: shlib.get_dependencies())
		if(ObjectFile *obj = dynamic_cast<ObjectFile *>(d))
			objs.push_back(obj);

	ExportDefinitions *exp = new ExportDefinitions(builder, *shlib.get_component(), objs);
	exp->set_tool(*this);

	ImportLibrary *imp = new ImportLibrary(builder, *shlib.get_component(), shlib);
	imp->add_dependency(*exp);
	imp->set_tool(*this);

	return imp;
}

Task *MingwDllTool::_run(const Target &target)
{
	const Tool &tool = *target.get_tool();

	const ImportLibrary *imp = dynamic_cast<const ImportLibrary *>(&target);
	const ExportDefinitions *exp = 0;
	if(imp)
		exp = &dynamic_cast<const ExportDefinitions &>(*imp->get_dependencies().front());
	else
		exp = dynamic_cast<const ExportDefinitions *>(&target);
	if(!imp && !exp)
		throw invalid_argument("MingwDllTool::run");

	vector<string> argv;
	argv.push_back(tool.get_executable()->get_path().str());

	/* dlltool is stupid and puts temporary files in the working directory by
	default */
	argv.push_back("--temp-prefix");
	char random[8];
	for(unsigned i=0; i<8; ++i)
		random[i] = 'a'+(rand()%26);
	argv.push_back(string("/tmp/")+string(random, 8));

	const Component &comp = *target.get_component();
	FS::Path work_dir = comp.get_package().get_source_directory();

	if(imp)
	{
		const SharedLibrary &shlib = *imp->get_shared_library();

		argv.push_back("-d");
		argv.push_back(relative(exp->get_path(), work_dir).str());

		argv.push_back("-D");
		argv.push_back(FS::basename(shlib.get_path()));

		argv.push_back("-l");
		argv.push_back(relative(imp->get_path(), work_dir).str());
	}
	else
	{
		for(Target *d: exp->get_dependencies())
			if(ObjectFile *obj = dynamic_cast<ObjectFile *>(d))
				argv.push_back(relative(obj->get_path(), work_dir).str());

		argv.push_back("-z");
		argv.push_back(relative(exp->get_path(), work_dir).str());
	}

	return new ExternalTask(argv, work_dir);
}
