#include <msp/builder/component.h>
#include <msp/builder/objectfile.h>
#include <msp/builder/sourcepackage.h>
#include <msp/builder/staticlibrary.h>
#include <msp/fs/utils.h>
#include "microsofttools.h"
#include "msvcarchiver.h"

using namespace std;
using namespace Msp;

MsvcArchiver::MsvcArchiver(Builder &b, const Architecture &a, const MicrosoftTools &m):
	Tool(b, &a, "AR"),
	ms_tools(m)
{
	input_suffixes.push_back(".obj");
	processing_unit = COMPONENT;
	set_command((ms_tools.get_vc_bin_dir()/"lib.exe").str(), false);
	set_run_external(_run);
}

Target *MsvcArchiver::create_target(const vector<Target *> &sources, const string &)
{
	if(sources.empty())
		throw invalid_argument("MsvcArchiver::create_target");

	vector<ObjectFile *> objs;
	objs.reserve(sources.size());
	for(Target *s: sources)
		objs.push_back(&dynamic_cast<ObjectFile &>(*s));

	const Component &comp = *objs.front()->get_component();
	StaticLibrary *lib = new StaticLibrary(builder, comp, objs);
	lib->set_tool(*this);
	return lib;
}

ExternalTask::Arguments MsvcArchiver::_run(const StaticLibrary &lib, FS::Path &work_dir)
{
	const Tool &tool = *lib.get_tool();

	vector<string> argv;
	argv.push_back(tool.get_executable()->get_path().str());
	argv.push_back("/NOLOGO");

	argv.push_back("/OUT:"+relative(lib.get_path(), work_dir).str());

	for(Target *d: lib.get_dependencies())
		if(ObjectFile *obj = dynamic_cast<ObjectFile *>(d))
			argv.push_back(relative(obj->get_path(), work_dir).str());

	return argv;
}
