#include <msp/builder/architecture.h>
#include <msp/builder/builder.h>
#include "microsofttools.h"
#include "msvcplugin.h"
#include "vcxprojectfile.h"
#include "vssolutionfile.h"

void MsvcPlugin::add_tools(Toolchain &toolchain, const Architecture &arch) const
{
	if(arch.get_system()=="windows")
		toolchain.add_toolchain(new MicrosoftTools(builder, arch));
}

void MsvcPlugin::create_targets(SourcePackage &spkg) const
{
	const Architecture &native_arch = builder.get_native_arch();

	if(native_arch.get_system()=="windows")
	{
		new VcxProjectFile(builder, spkg);
		new VsSolutionFile(builder, spkg);
	}
}


#ifdef MSVCTOOLS_BUILD
#if defined(_WIN32)
#define MSVCTOOLS_API __declspec(dllexport)
#elif defined(__GNUC__)
#define MSVCTOOLS_API __attribute__((visibility("default")))
#else
#define MSVCTOOLS_API
#endif

extern "C"
MSVCTOOLS_API Plugin *create_plugin(Builder &builder)
{
	return new MsvcPlugin(builder);
}
#endif
