#ifndef BINARYCOMPONENT_H_
#define BINARYCOMPONENT_H_

#include "component.h"
#include "libbuilder_api.h"

class FileTarget;
class Target;
class Tool;

class LIBBUILDER_API BinaryComponent: public Component
{
public:
	class Loader: public Msp::DataFile::DerivedObjectLoader<BinaryComponent, Component::Loader>
	{
	public:
		Loader(BinaryComponent &);
	private:
		void use(const std::string &);
	};

	enum Type
	{
		LIBRARY,
		PROGRAM,
		MODULE
	};

private:
	Type type;
	std::vector<const Component *> uses;

public:
	BinaryComponent(SourcePackage &p, const std::string &n, Type t): Component(p, n), type(t) { }

	Type get_type() const { return type; }

	void create_build_info() override;
	void update_exported_build_info(BuildInfo &) const override;
	void create_targets() const override;
private:
	std::vector<FileTarget *> create_sources() const;
	static std::vector<Target *> extract_group(std::vector<FileTarget *> &, const Tool &);
	void create_install(FileTarget &) const;
};

#endif
