#ifndef BINARYPACKAGE_H_
#define BINARYPACKAGE_H_

#include "libbuilder_api.h"
#include "package.h"

/**
Represents a package that is installed on the system, but can't be built by
Builder.
*/
class LIBBUILDER_API BinaryPackage: public Package
{
public:
	class Loader: public Msp::DataFile::DerivedObjectLoader<BinaryPackage, Package::Loader>
	{
	public:
		Loader(BinaryPackage &);
	private:
		void build_info();
		void header(const std::string &);
	};

	using Flags = std::vector<std::string>;

private:
	Msp::FS::Path base_path;
	std::vector<std::string> headers;
	BuildInfo static_binfo;

public:
	BinaryPackage(Builder &, const std::string &);

	const BuildInfo &get_static_build_info() const { return static_binfo; }

	static BinaryPackage *from_flags(Builder &, const std::string &, const Flags &, const Flags & = Flags());
private:
	static void process_flags(const Flags &, BuildInfo &);
	void do_prepare() override;
};

#endif
