#ifndef CHAINEDTASK_H_
#define CHAINEDTASK_H_

#include <vector>
#include "libbuilder_api.h"
#include "task.h"

/**
Runs multiple tasks as one unit, one after the other.  Execution of the chain
will stop if any of the component tasks terminates with an error.
*/
class LIBBUILDER_API ChainedTask: public Task
{
private:
	std::vector<Task *> tasks;
	unsigned current = 0;
	Status final_status = RUNNING;

public:
	ChainedTask(Task *t) { add_task(t); }
	~ChainedTask();

	void add_task(Task *);

	std::string get_command() const override;
	void start() override;
	Status check() override;
	Status wait() override;
private:
	bool process(Status);
};

#endif
