#ifndef INSTALLEDFILE_H_
#define INSTALLEDFILE_H_

#include "filetarget.h"
#include "libbuilder_api.h"
#include "sourcepackage.h"

/**
Represents the installation of a file.
*/
class LIBBUILDER_API InstalledFile: public FileTarget
{
private:
	FileTarget &source;
	Msp::FS::Path link;

public:
	InstalledFile(Builder &, const SourcePackage &, FileTarget &, const std::string & =std::string());
private:
	static Msp::FS::Path generate_target_path(const Msp::FS::Path &, const FileTarget &i, const std::string &);

public:
	const char *get_type() const override { return "InstalledFile"; }
	FileTarget &get_source() const { return source; }

	/** Sets a symlink for the file.  A relative path will be rooted at the
	directory the file resides in. */
	void set_symlink(const Msp::FS::Path &);

	const Msp::FS::Path &get_symlink() const { return link; }

	FileTarget *get_real_target() override;
private:
	void check_rebuild() override;

public:
	void clean() override;
};

#endif
