#include <msp/core/algorithm.h>
#include <msp/fs/utils.h>
#include "builder.h"
#include "component.h"
#include "objectfile.h"
#include "sourcefile.h"
#include "sourcepackage.h"

using namespace std;
using namespace Msp;

ObjectFile::ObjectFile(Builder &b, const Component &c, SourceFile &s):
	FileTarget(b, c.get_package(), generate_target_path(c, s.get_path())),
	source(s)
{
	component = &c;
	add_dependency(source);
}

FS::Path ObjectFile::generate_target_path(const Component &comp, const FS::Path &src)
{
	string fn = comp.flatten_source_path(src);
	const Architecture &arch = comp.get_package().get_builder().get_current_arch();
	return comp.get_temp_directory()/arch.create_filename<ObjectFile>(FS::basepart(fn));
}

void ObjectFile::set_used_in_shared_library(bool u)
{
	used_in_shlib = u;
}

void ObjectFile::collect_build_info(BuildInfo &binfo) const
{
	Target::collect_build_info(binfo);
	binfo.update_from(component->get_build_info_for_path(source.get_path()));
}
