#include <msp/strings/format.h>
#include "builder.h"
#include "package.h"

using namespace std;
using namespace Msp;

Package::Package(Builder &b, const string &n):
	builder(b),
	name(n),
	label(string(1, toupper(n[0]))+n.substr(1))
{
	builder.get_package_manager().add_package(this);
}

void Package::prepare()
{
	if(prepared)
		return;

	prepared = true;

	for(Package *r: required_pkgs)
	{
		r->prepare();
		broken |= r->is_broken();
	}

	do_prepare();
	broken |= !problems.empty();
}


Package::Loader::Loader(Package &p):
	DataFile::ObjectLoader<Package>(p),
	ArchitectureConditional(p.builder, p.name)
{
	add("label",   &Package::label);
	add("require", &Loader::require);
}

void Package::Loader::require(const string &n)
{
	Package *req = obj.builder.get_package_manager().find_package(n);
	if(req)
		obj.required_pkgs.push_back(req);
	else
		obj.problems.push_back(format("Required package %s not found", n));
}
