#include <stdexcept>
#include "pattern.h"

using namespace std;

Pattern::Pattern(const string &pat)
{
	string::size_type percent = pat.find('%');
	if(percent==string::npos)
		throw invalid_argument("No percent sign in pattern");
	prefix = pat.substr(0, percent);
	suffix = pat.substr(percent+1);
}

string Pattern::apply(const string &body) const
{
	string result = body;
	if(body.compare(0, prefix.size(), prefix))
		result = prefix+result;
	if(body.size()<=suffix.size() || body.compare(body.size()-suffix.size(), suffix.size(), suffix))
		result += suffix;
	return result;
}

vector<string> Pattern::apply_list(const vector<Pattern> &patterns, const string &body)
{
	vector<string> result;
	for(const Pattern &p: patterns)
		result.push_back(p.apply(body));
	return result;
}
