#ifndef PATTERN_H_
#define PATTERN_H_

#include <string>
#include <vector>
#include "libbuilder_api.h"

/**
Stores a filename pattern.  A pattern consists of a prefix and a suffix, and
can be applied to a body to form a complete filename.  Either or both of the
prefix and suffix may be empty.
*/
class LIBBUILDER_API Pattern
{
private:
	std::string prefix;
	std::string suffix;

public:
	/** Constructs a pattern from a single string.  The string must have exactly
	one percent sign (%) to separate the prefix and suffix. */
	Pattern(const std::string &);

	const std::string &get_prefix() const { return prefix; }
	const std::string &get_suffix() const { return suffix; }

	/** Applies the pattern to a body string. */
	std::string apply(const std::string &) const;

	/** Applies a list of patterns to the same body. */
	static std::vector<std::string> apply_list(const std::vector<Pattern> &, const std::string &);
};

#endif
