#include <msp/fs/utils.h>
#include <msp/strings/format.h>
#include "binarycomponent.h"
#include "builder.h"
#include "objectfile.h"
#include "sharedlibrary.h"
#include "sourcepackage.h"

using namespace std;
using namespace Msp;

SharedLibrary::SharedLibrary(Builder &b, const Msp::FS::Path &p):
	Binary(b, p),
	libname(FS::basepart(FS::basename(p)))
{
	const string &lib_prefix = builder.get_current_arch().get_patterns<SharedLibrary>().front().get_prefix();
	if(!libname.compare(0, lib_prefix.size(), lib_prefix))
		libname = libname.substr(lib_prefix.size());
}

SharedLibrary::SharedLibrary(Builder &b, const Component &c, const vector<ObjectFile *> &objs):
	Binary(b, c, generate_filename(c), objs),
	libname(c.get_name())
{
	const string &lib_prefix = builder.get_current_arch().get_patterns<SharedLibrary>().front().get_prefix();
	if(!libname.compare(0, lib_prefix.size(), lib_prefix))
		libname = libname.substr(lib_prefix.size());

	if(builder.get_current_arch().get_system()=="windows")
		install_location = "bin";
	else
		install_location = "lib";

	const BinaryComponent &bcomp = dynamic_cast<const BinaryComponent &>(*component);
	if(bcomp.get_type()==BinaryComponent::MODULE)
		install_location /= package->get_name();

	for(ObjectFile *o: objects)
		o->set_used_in_shared_library(true);
}

string SharedLibrary::generate_filename(const Component &comp)
{
	const BinaryComponent &bcomp = dynamic_cast<const BinaryComponent &>(comp);
	if(bcomp.get_type()==BinaryComponent::MODULE)
		return comp.get_name()+".dlm";
	else
	{
		const SourcePackage &pkg = comp.get_package();
		const Architecture &arch = pkg.get_builder().get_current_arch();
		const string &version = pkg.get_interface_version();
		if(!version.empty())
		{
			if(arch.get_system()=="windows")
				return arch.create_filename<SharedLibrary>(format("%s-%s", comp.get_name(), version));
			else if(arch.get_system()=="darwin")
				return arch.create_filename<SharedLibrary>(format("%s.%s", comp.get_name(), version));
			else
				return format("%s.%s", arch.create_filename<SharedLibrary>(comp.get_name()), version);
		}
		else
			return arch.create_filename<SharedLibrary>(comp.get_name());
	}
}

void SharedLibrary::set_import_library(ImportLibrary *imp)
{
	import_lib = imp;
}
